from django.contrib.contenttypes.fields import GenericRelation
from django.db import models
from django.utils.timezone import now


class TimeWizardInlineMixin(models.Model):
    """
    Mixin for the inline support of PeriodModel.
    """
    periods = GenericRelation(
        'time_wizard.PeriodModel',
    )

    class Meta:
        abstract = True

    @property
    def is_published(self):
        dt = now()
        for p in self.periods.all():
            if p.contains(dt):
                return True
        return False


class TimeWizardMixin(models.Model):
    """
    Mixin to let models have a foreign-key-relation to the TimeWizard. Property
    `is_published` is used to indicate if a TimeWizard is set wether or not
    to show the contents/children.
    """
    time_wizard = models.ForeignKey(
        'time_wizard.TimeWizardModel',
        on_delete=models.SET_NULL,
        blank=True,
        null=True,
    )

    class Meta:
        abstract = True

    @property
    def is_published(self):
        return self.time_wizard.is_published



import inspect
import sys

import holidays
from django.conf import settings

countries = []
country_provinces = {}
for i in inspect.getmembers(sys.modules['holidays'], inspect.isclass):
    if issubclass(i[1], holidays.HolidayBase) and len(i[0]) < 4:
        countries.append(i[0])
        provinces_states = getattr(i[1], 'PROVINCES', []) or \
            getattr(i[1], 'STATES', [])
        country_provinces.update({i[0]: provinces_states})


TIME_WIZARD_COUNTRIES = getattr(settings, 'TIME_WIZARD_COUNTRIES', countries)
TIME_WIZARD_COUNTRY_PROVINCES = getattr(settings,
                                        'TIME_WIZARD_COUNTRY_PROVINCES',
                                        country_provinces)



# Generated by Django 2.1.9 on 2019-07-06 06:23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('time_wizard', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='absoluteperiodmodel',
            name='periodmodel_ptr',
        ),
        migrations.RemoveField(
            model_name='holidayrangeperiodmodel',
            name='periodmodel_ptr',
        ),
        migrations.RemoveField(
            model_name='periodmodel',
            name='container',
        ),
        migrations.RemoveField(
            model_name='periodmodel',
            name='polymorphic_ctype',
        ),
        migrations.DeleteModel(
            name='AbsolutePeriodModel',
        ),
        migrations.DeleteModel(
            name='HolidayRangePeriodModel',
        ),
        migrations.DeleteModel(
            name='PeriodModel',
        ),
        migrations.DeleteModel(
            name='TimeWizardModel',
        ),
    ]



from django.conf.urls import url

from time_wizard import views

urlpatterns = [
    url(r'^load-holidays', views.load_holidays, name='ajax-load-holidays'),
    url(r'^load-provinces', views.load_provinces, name='ajax-load-provinces'),
]



from django.contrib import admin
from polymorphic.admin import (
    GenericStackedPolymorphicInline, PolymorphicInlineSupportMixin,
)

from time_wizard.models import (
    AbsolutePeriodModel, HolidayRangePeriodModel, PeriodModel, TimeWizardModel,
)


class PeriodModelInline(GenericStackedPolymorphicInline):
    class AbsolutePeriodModelInline(GenericStackedPolymorphicInline.Child):
        model = AbsolutePeriodModel

    class HolidayRangePeriodModelInline(GenericStackedPolymorphicInline.Child):
        model = HolidayRangePeriodModel

    model = PeriodModel
    child_inlines = (
        AbsolutePeriodModelInline,
        HolidayRangePeriodModelInline,
    )
    template = 'admin/edit_inline/holiday_stacked.html'


class TimeWizardModelAdmin(PolymorphicInlineSupportMixin, admin.ModelAdmin):
    model = TimeWizardModel
    inlines = [PeriodModelInline]


admin.site.register(TimeWizardModel, TimeWizardModelAdmin)



# workarounds.py contains workarounds and monkey-patches for external issues.


# TODO: Remove on release with bugfix
# Problem: problems with polymorphic and on_delete=CASCADE
# Issue: https://github.com/django-polymorphic/django-polymorphic/issues/
#        229#issuecomment-398434412
def NON_POLYMORPHIC_CASCADE(collector, field, sub_objs, using):
    from django.db.models import CASCADE
    sub_objs = sub_objs.non_polymorphic() \
        if hasattr(sub_objs, 'non_polymorphic') else sub_objs
    return CASCADE(collector, field, sub_objs, using)
